@@ -7,10 +7,10 @@
 //! * parallel data uploading
 extern crate vulkano;
 
-use std::collections::{BTreeMap, BTreeSet};
+use std::collections::BTreeMap;
 use std::fmt::Debug;
 use std::sync::mpsc::{Receiver, Sender};
-use std::sync::{Arc, Mutex};
+use std::sync::Arc;
 
 use crate::shader::create_pipeline;
 use epi::egui;
@@ -21,33 +21,34 @@ use vulkano::buffer::cpu_pool::CpuBufferPoolChunk;
 use vulkano::buffer::{BufferUsage, CpuBufferPool};
 use vulkano::command_buffer::pool::standard::StandardCommandPoolAlloc;
 use vulkano::command_buffer::{
-    CommandBufferExecFuture, CommandBufferUsage, DynamicState, PrimaryAutoCommandBuffer,
-    SubpassContents,
+    CommandBufferUsage, DynamicState, PrimaryAutoCommandBuffer, SubpassContents,
 };
-use vulkano::descriptor::descriptor_set::PersistentDescriptorSet;
-use vulkano::descriptor::DescriptorSet;
+use vulkano::descriptor_set::persistent::PersistentDescriptorSet;
+use vulkano::descriptor_set::DescriptorSet;
 use vulkano::device::{Device, Queue};
 use vulkano::image::view::{ImageView, ImageViewAbstract, ImageViewCreationError};
+#[cfg(feature = "depth_rendering_mode")]
+use vulkano::image::ImageAccess;
 use vulkano::image::{
-    AttachmentImage, ImageAccess, ImageCreationError, ImageDimensions, ImageUsage, MipmapsCount,
-    SwapchainImage,
+    AttachmentImage, ImageCreationError, ImageDimensions, ImageUsage, MipmapsCount, SwapchainImage,
 };
 use vulkano::memory::pool::StdMemoryPool;
-use vulkano::pipeline::vertex::SingleBufferDefinition;
+use vulkano::pipeline::vertex::BuffersDefinition;
 use vulkano::pipeline::viewport::{Scissor, Viewport};
 use vulkano::pipeline::{GraphicsPipeline, GraphicsPipelineAbstract};
 use vulkano::render_pass::{Framebuffer, FramebufferAbstract};
 use vulkano::sampler::{Filter, MipmapMode, Sampler, SamplerAddressMode};
 use vulkano::swapchain::SwapchainAcquireFuture;
-use vulkano::sync::{GpuFuture, NowFuture};
+use vulkano::sync::GpuFuture;
 
 mod render_pass;
 
 mod shader;
 /*
-same layout as EGUI
+same layout as egui
 memory layout is specified by Vertex trait
-rust's orphan rule we can't use egui::Vertex directly
+rust's orphan rule we can't use egui::Vertex directly ie . we can't do this
+vulkano::impl_vertex!(egui::epaint::Vertex,pos,uv,color);
 so we can cast from Vertex and send it to GPU directly
 */
 #[repr(C)]
@@ -89,7 +90,7 @@ impl ScreenDescriptor {
     }
 }
 
-type Pipeline = GraphicsPipeline<SingleBufferDefinition<EguiVulkanoVertex>>;
+type Pipeline = GraphicsPipeline<BuffersDefinition>;
 struct RenderResource {
     index_buffer: CpuBufferPoolChunk<u32, Arc<StdMemoryPool>>,
     vertex_buffer: CpuBufferPoolChunk<EguiVulkanoVertex, Arc<StdMemoryPool>>,
@@ -105,15 +106,11 @@ pub struct EguiVulkanoRenderPass {
     device: Arc<Device>,
     queue: Arc<Queue>,
     frame_buffers: Vec<Arc<dyn FramebufferAbstract + Send + Sync>>,
-    depth_buffer: Arc<dyn ImageViewAbstract + Send + Sync>,
     vertex_buffer_pool: CpuBufferPool<EguiVulkanoVertex>,
     index_buffer_pool: CpuBufferPool<u32>,
     image_staging_buffer: CpuBufferPool<u8>,
     sampler_desc_set: Arc<dyn DescriptorSet + Send + Sync>,
     //set=0 binding=0
-    image_transfer_requests: BTreeSet<Option<u64>>,
-    request_sender: Sender<Work>,
-    done_notifier: Receiver<Option<u64>>,
     thread_pool: ThreadPool,
     render_resource_tx: Sender<RenderResource>,
     render_resource_rx: Receiver<RenderResource>,
@@ -125,24 +122,6 @@ fn texture_id_as_option_u64(x: TextureId) -> Option<u64> {
         TextureId::User(id) => Some(id),
     }
 }
-
-type Work = (
-    Option<u64>,
-    CommandBufferExecFuture<NowFuture, PrimaryAutoCommandBuffer>,
-);
-
-fn spawn_image_upload_thread() -> (Sender<Work>, Receiver<Option<u64>>) {
-    let (tx, rx): (Sender<Work>, Receiver<Work>) = std::sync::mpsc::channel();
-    let (notifier, getter) = std::sync::mpsc::channel();
-    let rx = Mutex::new(rx);
-    std::thread::spawn(move || {
-        for request in rx.lock().unwrap().iter() {
-            request.1.flush().unwrap();
-            notifier.send(request.0).unwrap();
-        }
-    });
-    (tx, getter)
-}
 /*
 We use Option<u64> as TextureId
 TextureId does not implement Ord trait
@@ -173,34 +152,22 @@ impl EguiVulkanoRenderPass {
             0.0,
             0.0,
         )
-        .unwrap();
+        .expect("failed to create sampler");
         let thread_pool = ThreadPool::new(num_cpus::get());
 
         let vertex_buffer_pool = CpuBufferPool::vertex_buffer(device.clone());
         let index_buffer_pool = CpuBufferPool::new(device.clone(), BufferUsage::index_buffer());
         let image_staging_buffer =
             CpuBufferPool::new(device.clone(), BufferUsage::transfer_source());
-        let (request_sender, done_notifier) = spawn_image_upload_thread();
         let sampler_desc_set = Arc::new(
-            PersistentDescriptorSet::start(
-                pipeline.layout().descriptor_set_layout(0).unwrap().clone(),
-            )
-            .add_sampler(sampler)
-            .unwrap()
-            .build()
-            .unwrap(),
+            PersistentDescriptorSet::start(pipeline.layout().descriptor_set_layouts()[0].clone())
+                .add_sampler(sampler)
+                .unwrap()
+                .build()
+                .unwrap(),
         ) as Arc<dyn DescriptorSet + Send + Sync>;
         let (buffers_tx, buffers_rx) = std::sync::mpsc::channel();
-        let depth_buffer = ImageView::new(
-            AttachmentImage::with_usage(
-                device.clone(),
-                [1, 1],
-                vulkano::format::Format::D32Sfloat,
-                ImageUsage::transient_depth_stencil_attachment(),
-            )
-            .unwrap(),
-        )
-        .unwrap() as Arc<dyn ImageViewAbstract + Send + Sync>;
+
         Self {
             pipeline,
             egui_texture_version: None,
@@ -208,15 +175,10 @@ impl EguiVulkanoRenderPass {
             device,
             queue,
             frame_buffers: vec![],
-            depth_buffer,
             vertex_buffer_pool,
             index_buffer_pool,
             image_staging_buffer,
             sampler_desc_set,
-
-            image_transfer_requests: BTreeSet::new(),
-            request_sender,
-            done_notifier,
             thread_pool,
             render_resource_tx: buffers_tx,
             render_resource_rx: buffers_rx,
@@ -228,20 +190,14 @@ impl EguiVulkanoRenderPass {
         &mut self,
         swap_chain_images: &[Arc<SwapchainImage<Wnd>>],
     ) {
-        self.resize_depth_buffer(swap_chain_images[0].dimensions().width_height());
         self.frame_buffers = swap_chain_images
             .iter()
             .map(|image| {
                 let view = ImageView::new(image.clone()).unwrap();
-                Arc::new(
-                    Framebuffer::start(self.pipeline.clone().render_pass().clone())
-                        .add(view)
-                        .unwrap()
-                        .add(self.depth_buffer.clone())
-                        .unwrap()
-                        .build()
-                        .unwrap(),
-                ) as Arc<dyn FramebufferAbstract + Send + Sync>
+                let fb = Framebuffer::start(self.pipeline.render_pass().clone())
+                    .add(view)
+                    .unwrap();
+                Arc::new(fb.build().unwrap()) as Arc<dyn FramebufferAbstract + Send + Sync>
             })
             .collect::<Vec<_>>();
     }
@@ -273,16 +229,10 @@ impl EguiVulkanoRenderPass {
     ) -> PrimaryAutoCommandBuffer<StandardCommandPoolAlloc> {
         let framebuffer = match render_target {
             RenderTarget::ColorAttachment(color_attachment) => {
-                self.resize_depth_buffer(color_attachment.image().dimensions().width_height());
-                Arc::new(
-                    vulkano::render_pass::Framebuffer::start(self.pipeline.render_pass().clone())
-                        .add(color_attachment)
-                        .unwrap()
-                        .add(self.depth_buffer.clone())
-                        .unwrap()
-                        .build()
-                        .expect("[BackEnd] failed to create frame buffer"),
-                )
+                let fb = Framebuffer::start(self.pipeline.render_pass().clone())
+                    .add(color_attachment)
+                    .unwrap();
+                Arc::new(fb.build().expect("[BackEnd] failed to create frame buffer"))
             }
             RenderTarget::FrameBufferIndex(id) => self.frame_buffers[id].clone(),
         };
@@ -298,11 +248,7 @@ impl EguiVulkanoRenderPass {
             .expect("[BackEnd] failed to create command buffer builder");
 
         command_buffer_builder
-            .begin_render_pass(
-                framebuffer,
-                SubpassContents::Inline,
-                vec![[0.0, 0.0, 0.0, 0.0].into(), 1.0.into()],
-            )
+            .begin_render_pass(framebuffer, SubpassContents::Inline, vec![[0.0; 4].into()])
             .unwrap();
 
         let physical_height = screen_descriptor.physical_height;
@@ -323,7 +269,7 @@ impl EguiVulkanoRenderPass {
         #[cfg(feature = "backend_debug")]
         println!("[BackEnd] start frame");
 
-        #[cfg(feature = "backend_debug")]
+        #[cfg(all(feature = "backend_debug", not(feature = "depth_rendering_mode")))]
         let instant = std::time::Instant::now();
         let mut job_count = 0;
 
@@ -342,7 +288,7 @@ impl EguiVulkanoRenderPass {
             let vertex_buffer_pool = self.vertex_buffer_pool.clone();
 
             #[cfg(feature = "backend_debug")]
-            println!("Fission");
+            println!("[BackEnd] Fission");
             #[cfg(feature = "backend_debug")]
             println!(
                 "[BackEnd] mesh {}  vertices {} indices {} texture_id {:?}",
@@ -351,9 +297,11 @@ impl EguiVulkanoRenderPass {
                 indices.len(),
                 texture_id
             );
-            // safe
-            // use same memory layout
-            // GPU send only cast
+            /*
+            safe
+            use same memory layout
+            GPU send only cast
+            */
             let vertex_slice: Box<[EguiVulkanoVertex]> = unsafe { std::mem::transmute(vertices) };
             let tx = self.render_resource_tx.clone();
             /*Data uploading*/
@@ -373,10 +321,6 @@ impl EguiVulkanoRenderPass {
             });
             job_count += 1;
         }
-        #[cfg(feature = "backend_debug")]
-        println!("[BackEnd] upload finished in {:?}", instant.elapsed());
-        #[cfg(feature = "backend_debug")]
-        println!("Fusion {} meshes", job_count);
 
         let draw = |render_resource: RenderResource| {
             let texture_id = render_resource.texture_id;
@@ -386,21 +330,7 @@ impl EguiVulkanoRenderPass {
                 TextureId::User(_) => IS_USER,
                 _ => IS_EGUI,
             };
-
-            let layer = {
-                #[cfg(feature = "depth_rendering_mode")]
-                {
-                    ((render_resource.job_id as f32 / job_count as f32)
-                        * std::f32::consts::FRAC_PI_2)
-                        .cos()
-                }
-                #[cfg(not(feature = "depth_rendering_mode"))]
-                {
-                    0.0
-                }
-            };
-            #[cfg(feature = "backend_debug")]
-            println!("layer {} z {}", render_resource.job_id, layer);
+            let layer = 0.0;
             let pc = PushConstants {
                 u_screen_size: [logical.0 as f32, logical.1 as f32],
                 marker,
@@ -417,32 +347,23 @@ impl EguiVulkanoRenderPass {
                         render_resource.index_buffer,
                         (self.sampler_desc_set.clone(), texture),
                         pc,
-                        vec![],
                     )
                     .unwrap();
             }
         };
-        #[cfg(feature = "depth_rendering_mode")]
-        {
-            let _: Vec<()> = self
-                .render_resource_rx
-                .iter()
-                .take(job_count)
-                .map(draw)
-                .collect();
-        }
 
-        #[cfg(not(feature = "depth_rendering_mode"))]
         {
             /*
                 To avoid image flicker , We sort by job id
-            */
-            /*
                 We must take job_counts commands to avoid blocking
             */
 
             let mut jobs: Vec<RenderResource> =
                 self.render_resource_rx.iter().take(job_count).collect();
+            #[cfg(feature = "backend_debug")]
+            println!("[BackEnd] upload finished in {:?}", instant.elapsed());
+            #[cfg(feature = "backend_debug")]
+            println!("Fusion {} meshes", job_count);
             jobs.sort_by_key(|render_resource| render_resource.job_id);
             let _: Vec<()> = jobs.into_iter().map(draw).collect();
         }
@@ -454,47 +375,31 @@ impl EguiVulkanoRenderPass {
     /// Update egui system texture
     /// You must call before every  create_command_buffer when drawing content changed
     pub fn request_upload_egui_texture(&mut self, texture: &Texture) {
-        //no change
-        if self.egui_texture_version == Some(texture.version) {
-            return;
-        }
-        let staging_sub_buffer = self
-            .image_staging_buffer
-            .chunk(texture.pixels.iter().copied())
-            .unwrap();
-        let image = vulkano::image::ImmutableImage::from_buffer(
-            staging_sub_buffer,
-            ImageDimensions::Dim2d {
-                width: texture.width as u32,
-                height: texture.height as u32,
-                array_layers: 1,
-            },
-            MipmapsCount::One,
-            vulkano::format::Format::R8Unorm,
-            self.queue.clone(),
+        assert!(
+            (texture.width * texture.height) >= texture.pixels.len(),
+            "Invalid pixels in EGUI system texture given {} x {} actual {} pixels",
+            texture.width,
+            texture.height,
+            texture.pixels.len()
+        );
+        self.set_texture(
+            TextureId::Egui,
+            &texture.pixels,
+            (texture.width as u32, texture.height as u32),
+            Some(texture.version),
         )
-        .unwrap();
-        self.request_sender
-            .send((None, image.1))
-            .expect("[BackEnd] failed to send system texture upload request");
-        let image_view = ImageView::new(image.0).unwrap();
-        let image_view = image_view as Arc<dyn ImageViewAbstract + Sync + Send>;
-        let pipeline = self.pipeline.clone();
-        let texture_desc = Self::create_descriptor_set_from_view(pipeline, image_view);
-        self.egui_textures
-            .insert(None, Some(TextureDescriptor(texture_desc)));
-        self.egui_texture_version = Some(texture.version);
     }
-
+    #[inline(always)]
     fn alloc_user_texture(&mut self) -> TextureId {
-        /*
-        if we find empty slot
-        */
-        for (i, tex) in self.egui_textures.iter() {
-            if tex.is_none() && i.is_some() {
-                return TextureId::User(i.unwrap());
-            }
-        }
+        //if we find empty slot
+        //refuse egui system texture
+        if let Some(empty_slot) = self
+            .egui_textures
+            .iter()
+            .find(|entry| entry.0.is_some() && entry.1.is_none())
+        {
+            return TextureId::User(empty_slot.0.unwrap());
+        };
         /*
                 Compatibility for glium backend .
             Because glium's TextureId start from 0.
@@ -507,106 +412,97 @@ impl EguiVulkanoRenderPass {
         self.egui_textures.insert(Some(id as u64), None);
         TextureId::User(id as u64)
     }
+    #[inline(always)]
     fn free_user_texture(&mut self, id: TextureId) {
         let t_id = texture_id_as_option_u64(id);
         self.egui_textures
             .get_mut(&t_id)
             .and_then(|option| option.take());
     }
-    /// waiting image upload done.
-    ///
-    /// usually you don't need to call .
-    ///
-    /// this cause blocking but ensure no image glitch.
-    ///
-    /// you must call before  create command buffer
-    pub fn wait_texture_upload(&mut self) {
-        //wait
-        // check request is not empty
-        if !self.image_transfer_requests.is_empty() {
-            for done in self.done_notifier.iter() {
-                #[cfg(feature = "backend_debug")]
-                println!("[BackEnd] image upload finished : {:?}", done);
-                self.image_transfer_requests.remove(&done);
-                if self.image_transfer_requests.is_empty() {
-                    break;
-                }
-            }
-        }
-    }
+    #[inline(always)]
     fn create_descriptor_set_from_view(
         pipeline: Arc<Pipeline>,
         image_view: Arc<dyn ImageViewAbstract + Sync + Send>,
     ) -> Arc<dyn DescriptorSet + Send + Sync> {
         Arc::new(
-            PersistentDescriptorSet::start(
-                pipeline.layout().descriptor_set_layout(1).unwrap().clone(),
-            )
-            .add_image(image_view)
-            .unwrap()
-            .build()
-            .unwrap(),
+            PersistentDescriptorSet::start(pipeline.layout().descriptor_set_layouts()[1].clone())
+                .add_image(image_view)
+                .unwrap()
+                .build()
+                .unwrap(),
         )
     }
+    #[inline(always)]
     fn get_descriptor_set(&self, texture_id: TextureId) -> Arc<dyn DescriptorSet + Send + Sync> {
         let t_id = texture_id_as_option_u64(texture_id);
         self.egui_textures
             .get(&t_id)
-            .unwrap()
+            .expect("[BackEnd] invalid Texture ID given")
             .as_ref()
-            .unwrap()
+            .expect("[BackEnd] access to freed image")
             .0
             .clone()
     }
+    #[inline(always)]
+    fn set_texture(
+        &mut self,
+        id: TextureId,
+        pixels: &[u8],
+        size: (u32, u32),
+        version: Option<u64>,
+    ) {
+        let format = match id {
+            TextureId::Egui => {
+                if self.egui_texture_version == version {
+                    return;
+                }
+                self.egui_texture_version = version;
+                vulkano::format::Format::R8Unorm
+            }
+            TextureId::User(_) => vulkano::format::Format::R8G8B8A8Srgb,
+        };
+        let staging_sub_buffer = self
+            .image_staging_buffer
+            .chunk(pixels.iter().copied())
+            .unwrap();
+
+        let (image, future) = vulkano::image::ImmutableImage::from_buffer(
+            staging_sub_buffer,
+            ImageDimensions::Dim2d {
+                width: size.0,
+                height: size.1,
+                array_layers: 1,
+            },
+            MipmapsCount::One,
+            format,
+            self.queue.clone(),
+        )
+        .unwrap();
+        let t_id = texture_id_as_option_u64(id);
+        self.thread_pool.execute(move || future.flush().unwrap());
+        let image_view = ImageView::new(image).unwrap() as Arc<dyn ImageViewAbstract + Sync + Send>;
+        let pipeline = self.pipeline.clone();
+        let texture_desc = Self::create_descriptor_set_from_view(pipeline, image_view);
+        self.egui_textures
+            .insert(t_id, Some(TextureDescriptor(texture_desc)));
+    }
     pub fn set_user_texture(
         &mut self,
         id: TextureId,
         size: (usize, usize),
         srgba_pixels: &[Color32],
     ) {
-        #[cfg(feature = "backend_debug")]
-        println!("new texture arrived {:?}", id);
-        let t_id = texture_id_as_option_u64(id);
-        let image_staging_buffer = self.image_staging_buffer.clone();
-        let pipeline = self.pipeline.clone();
-        let queue = self.queue.clone();
-        let sender = self.request_sender.clone();
-        let is_executed =
-            self.egui_textures
-                .get_mut(&t_id)
-                .map(|slot: &mut Option<TextureDescriptor>| {
-                    // SAFETY: GPU send only cast
-                    let pixels = unsafe {
-                        std::slice::from_raw_parts(
-                            srgba_pixels.as_ptr() as *const u8,
-                            srgba_pixels.len() * 4,
-                        )
-                    };
-                    let sub_buffer = image_staging_buffer.chunk(pixels.iter().copied()).unwrap();
-                    let (image, future) = vulkano::image::ImmutableImage::from_buffer(
-                        sub_buffer,
-                        ImageDimensions::Dim2d {
-                            width: size.0 as u32,
-                            height: size.1 as u32,
-                            array_layers: 1,
-                        },
-                        MipmapsCount::One,
-                        vulkano::format::Format::R8G8B8A8Srgb,
-                        queue,
-                    )
-                    .unwrap();
-                    sender
-                        .send((t_id, future))
-                        .expect("failed to send image copy request");
-
-                    let image_view = ImageView::new(image).unwrap();
-                    let image_view = image_view as Arc<dyn ImageViewAbstract + Sync + Send>;
-                    let desc = Self::create_descriptor_set_from_view(pipeline, image_view);
-                    slot.replace(TextureDescriptor(desc));
-                });
-        if is_executed.is_some() {
-            self.image_transfer_requests.insert(t_id);
-        }
+        assert!(
+            (size.0 * size.1) >= srgba_pixels.len(),
+            "Pixel count was invalid given {} x {} actual pixels {}",
+            size.0,
+            size.1,
+            srgba_pixels.len()
+        );
+        let pixels = unsafe {
+            std::slice::from_raw_parts(srgba_pixels.as_ptr() as *const u8, srgba_pixels.len() * 4)
+        };
+        self.set_texture(id, pixels, (size.0 as u32, size.1 as u32), None);
     }
     /// register vulkano image view as egui texture
     ///
@@ -690,20 +586,7 @@ impl EguiVulkanoRenderPass {
             Err(err) => Err(InitRenderAreaError::ImageCreationError(err)),
         }
     }
-    fn resize_depth_buffer(&mut self, dimensions: [u32; 2]) {
-        if self.depth_buffer.image().dimensions().width_height() != dimensions {
-            self.depth_buffer = ImageView::new(
-                AttachmentImage::with_usage(
-                    self.device.clone(),
-                    dimensions,
-                    vulkano::format::Format::D32Sfloat,
-                    ImageUsage::transient_depth_stencil_attachment(),
-                )
-                .unwrap(),
-            )
-            .unwrap() as Arc<dyn ImageViewAbstract + Send + Sync>;
-        }
-    }
+
     ///  recreate vulkano texture.
     ///
     ///  usable for render target resize.
@@ -815,7 +698,7 @@ fn calc_scissor(
     let height = clip_max_y - clip_min_y;
     #[cfg(feature = "backend_debug")]
     println!(
-        "clip_rectangle min_x : {} min_y : {} width :{} height : {} ",
+        "[BackEnd] clip_rectangle min_x : {} min_y : {} width :{} height : {} ",
         clip_min_x, clip_min_y, width, height
     );
     if (width == 0) | (height == 0) {
